import cliutils, docopt, logging, jester, json, os, sequtils, strutils, tables

import personal_measure_apipkg/api
import personal_measure_apipkg/configuration
import personal_measure_apipkg/service
import personal_measure_apipkg/version

# temp for testing
import personal_measure_apipkg/db
import personal_measure_apipkg/models
import bcrypt

const DEFAULT_CONFIG = PMApiConfig(
  authSecret: "change me",
  dbConnString: "",
  debug: false,
  port: 8080,
  pwdCost: 11)

proc loadConfig*(args: Table[string, docopt.Value] = initTable[string, docopt.Value]()): PMApiConfig  =

  let filePath =
      if args["--config"]: $args["--config"]
      else: "personal_measure_api.config.json"

  var json: JsonNode
  try: json = parseFile(filePath)
  except:
    json = %DEFAULT_CONFIG
    if not existsFile(filePath):
      info "created new configuration file \"" & filePath & "\""
      filePath.writeFile($json)
    else:
      warn "Cannot read configuration file \"" & filePath & "\":\n\t" &
        getCurrentExceptionMsg()

  let cfg = CombinedConfig(docopt: args, json: json)

  result = PMApiConfig(
    authSecret: cfg.getVal("authSecret"),
    dbConnString: cfg.getVal("dbConnString"),
    debug: "true".startsWith(cfg.getVal("debug", "false").toLower()),
    port: parseInt(cfg.getVal("port", "8080")),
    pwdCost: cast[int8](parseInt(cfg.getVal("pwdCost", "11"))))
    
proc initContext(args: Table[string, docopt.Value]): PMApiContext =

  var cfg: PMApiConfig
  var db: PMApiDb

  try: cfg = loadConfig(args)
  except: raiseEx "Unable to load configuration: \n\t" & getCurrentExceptionMsg()

  try: db = connect(cfg.dbConnString)
  except: raiseEx "Unable to connect to the database:\n\t" & getCurrentExceptionMsg()

  result = PMApiContext(cfg: cfg, db: db)

when isMainModule:

  try:
    let doc = """
Usage:
  personal_measure_api test [options]
  personal_measure_api serve [options]
  personal_measure_api hashpwd <password> [options]
  personal_measure_api create-admin <email> <password>

Options:

  -C, --config <cfgFile>    Location of the config file (defaults to personal_measure.config.json)
  --salt <salt>             Use a given salt for password hashing
  --pwdCost <cost>             Specify the cost for password hashing
"""
    logging.addHandler(newConsoleLogger())

    # Initialize our service context
    let args = docopt(doc, version = PM_API_VERSION)
    let ctx = initContext(args)

    if args["create-admin"]:
      let allUsers = ctx.db.getAllUsers();
      if allUsers.anyIt(it.isAdmin):
        raise newException(Exception, "admin user already exists, refusing to create a new admin")

      let pwdAndSalt = hashPwd($args["<password>"], ctx.cfg.pwdCost)

      let adminUser = User(
        displayName: "Admin",
        email: $args["<email>"],
        hashedPwd: pwdAndSalt.pwd,
        salt: pwdAndSalt.salt,
        isAdmin: true)

      echo ctx.db.createUser(adminUser)

    if args["hashpwd"]:
      if args["--salt"]:
        echo hashWithSalt($args["<password>"], $args["--salt"])
      else:
        let cost = ctx.cfg.pwdCost
        echo hashPwd($args["<password>"], cast[int8](cost))

    if args["test"]:
      echo "test"
      echo ctx.db.findUsersByEmail("jonathan@jdbernard.com")
      echo genSalt(ctx.cfg.pwdCost)

    if args["serve"]: start(ctx)

  except:
    fatal "pit: " & getCurrentExceptionMsg()
    #raise getCurrentException()
    quit(QuitFailure)
