import Vue from 'vue';
import { default as Router, Route } from 'vue-router';

import Login from '@/views/Login.vue';
import Measures from '@/views/Measures.vue';
import UserAccount from '@/views/UserAccount.vue';
import QuickPanels from '@/views/QuickPanels.vue';
import { authStore } from '@/store';

Vue.use(Router);

const router = new Router({
  mode: 'history',
  base: process.env.BASE_URL,
  routes: [
    {
      path: '/',
      redirect: '/measures'
    },
    {
      path: '/login',
      name: 'login',
      component: Login
    },
    {
      path: '/measures',
      name: 'measures',
      component: Measures,
      meta: { requiresAuth: true }
    },
    {
      path: '/user-account',
      name: 'user-account',
      component: UserAccount,
      meta: { requiresAuth: true }
    },
    {
      path: '/quick-panels',
      name: 'quick-panels',
      component: QuickPanels,
      meta: { requiresAuth: true }
    }
  ]
});

// Auth filter
router.beforeEach((to, from, next) => {
  if (to.matched.some((record) => record.meta.requiresAuth)) {
    if (!authStore.authToken) {
      next({ name: 'login' });
        // params: { redirect: to.path } });
    } else { next(); } // if authed
  } else { next(); } // if auth required
});

export default router;
