import {
  Action,
  Module,
  Mutation,
  MutationAction,
  VuexModule
} from 'vuex-module-decorators';
import api from '@/services/pm-api-client';
import { logService } from '@/services/logging';
import { ApiToken } from '@/models';

const log = logService.getLogger('/store-modules/api-tokens');

@Module({ namespaced: true, name: 'apiToken' })
export class ApiTokenStoreModule extends VuexModule {
  public apiTokens: ApiToken[] = [];

  @MutationAction({ mutate: ['apiTokens'] })
  public async fetchAllApiTokens() {
    return { apiTokens: await api.getAllApiTokens() };
  }

  @MutationAction({ mutate: ['apiTokens'] })
  public async createApiToken(newToken: ApiToken) {
    return { apiTokens: await api.createApiToken(newToken) };
  }

  @MutationAction({ mutate: ['apiTokens'] })
  public async deleteApiToken(tokenId: string) {
    api.deleteApiToken(tokenId);
    return { apiTokens: this.apiTokens.filter((tok) => tok.id !== tokenId) };
  }
}
