import {
  Action,
  Module,
  Mutation,
  VuexModule
} from 'vuex-module-decorators';
import Cookies from 'js-cookie';
import { LoginSubmit } from '@/models';
import api from '@/services/pm-api-client';
import { logService } from '@/services/logging';
import { userStore, measureStore } from '@/store';

const COOKIE_TOKEN = 'pm-api-cookie-token';

const log = logService.getLogger('/store-modules/auth');

@Module({ namespaced: true, name: 'auth' })
export class AuthStoreModule extends VuexModule {
  public authToken: string | null = null;

  @Action({ commit: 'SET_TOKEN', rawError: true })
  public async login(creds: LoginSubmit) {
    const authToken = await api.createAuthToken(creds); // API will cache this token
    Cookies.set(COOKIE_TOKEN, authToken, { secure: true });
    await userStore.fetchUser();
    await measureStore.fetchAllMeasures();
    log.trace('User login successful.');
    return authToken;
  }

  @Action({ commit: 'SET_TOKEN', rawError: true })
  public async findLocalToken() {
    const token = Cookies.get(COOKIE_TOKEN);
    if (token) { return token; }
    else { throw new Error('No auth token stored as a cookie.'); }
  }

  @Mutation private SET_TOKEN(t: string) {
    this.authToken = t;
  }
}
